package edu.buffalo.cse.cse462.dao;

import java.sql.Date;

import java.util.Set;

import edu.buffalo.cse.cse462.model.Customer;
import edu.buffalo.cse.cse462.model.Order;
import edu.buffalo.cse.cse462.model.Product;

/**
 * OrderDAO is a GenericDAO for Order objects with Integer keys.
 * 
 * Important: Order objects loaded from the database (in any find method) must 
 * load the associated OrderEntry objects. However, when updating or deleting 
 * an Order, you do not need to delete or update associated OrderEntry objects
 * as this is taken care automatically by the RDBMS (see the corresponding 
 * CREATE TABLE statements).
 */
public interface OrderDAO extends GenericDAO<Order, Integer> {

  /**
   * Given a customer, returns the set of all orders placed by the customer.
   */
  public Set<Order> findByCustomer(Customer customer) throws DAOException;

  /**
   * Given a range of dates, returns the set of all orders placed as follows:
   * 
   *   a) if maxDate is NULL, all orders placed after or on minDate;
   *   b) if minDate is NULL, all orders placed before or on maxDate;
   *   c) otherwise, all orders place between minDate and maxDate (inclusive).
   * 
   */
  public Set<Order> findByDate(Date minDate, Date maxDate) throws DAOException;

  /**
   * Given a product, returns the set of all orders containing an entry with such product.
   */
  public Set<Order> findByProduct(Product product) throws DAOException;

  /**
   * Given a set of products, returns the set of all orders containing an entry with each product.
   */
  public Set<Order> findByProducts(Set<Product> products) throws DAOException;

  /**
   * Given a range of values, returns the set of all orders with gross value as follows:
   * 
   *   a) if maxValue is NULL, all orders with gross value greater or equal to minValue;
   *   b) if minValue is NULL, all orders with gross value smaller or equal to maxValue;
   *   c) otherwise, all orders with gross value between minValue and maxValue (inclusive).
   * 
   */
  public Set<Order> findByValue(Double minValue, Double maxValue) throws DAOException;
}