package edu.buffalo.cse.cse462.internal.dao.oracle;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import edu.buffalo.cse.cse462.dao.CustomerDAO;
import edu.buffalo.cse.cse462.dao.DAOException;
import edu.buffalo.cse.cse462.internal.dao.JDBCSession;
import edu.buffalo.cse.cse462.model.Customer;
import edu.buffalo.cse.cse462.model.impl.CustomerImpl;

/**
 * OracleCustomerDAO is a CustomerDAO that relies on a JDBCSession for connection to an Oracle
 * database.
 * 
 */
public class OracleCustomerDAO implements CustomerDAO {

  // identifiers for the parameters to PreparedStatement objects
  private final int ID_CID = 1;
  private final int ID_CITY = 2;
  private final int ID_DOB = 3;
  private final int ID_FIRST_NAME = 4;
  private final int ID_LAST_NAME = 5;
  private final int ID_STATE = 6;

  // SQL statements used by the methods
  private final String DELETE = "DELETE FROM Customer WHERE cid = ?";
  private final String INSERT = "INSERT INTO Customer(cid, city, dob, firstName, lastName, state) VALUES(?,?,?,?,?,?)";
  private final String READ = "SELECT cid, city, dob, firstName, lastName, state FROM Customer WHERE cid = ?";

  // shared JDBC session
  private final JDBCSession session;

  // the constructor receives a JDBCSession from the OracleDAOSession
  public OracleCustomerDAO(JDBCSession session) {

    this.session = session;
  }

  /*
   * (non-Javadoc)
   * @see edu.buffalo.cse.cse462.dao.GenericDAO#insert(java.lang.Object)
   */
  @Override
  public boolean insert(Customer instance) throws DAOException {

    // the connection is shared by the session, do not close it
    Connection conn = session.getConnection();
    try {
      // this prepared statement is local, make sure you close it
      PreparedStatement stmt = conn.prepareStatement(INSERT);
      try {
        stmt.setInt(ID_CID, instance.getCid());
        stmt.setString(ID_CITY, instance.getCity());
        if (instance.getDateOfBirth() == null) {
          stmt.setNull(ID_DOB, java.sql.Types.DATE);
        }
        else {
          stmt.setDate(ID_DOB, instance.getDateOfBirth());
        }
        stmt.setString(ID_FIRST_NAME, instance.getFirstName());
        stmt.setString(ID_LAST_NAME, instance.getLastName());
        stmt.setString(ID_STATE, instance.getState());
        return stmt.executeUpdate() == 1; // exactly one tuple affected?
      }
      finally {
        stmt.close();
        stmt = null;
      }
    }
    catch (SQLException e) {
      throw new DAOException("Could not execute insert statement for customer.", e);
    }
  }

  /* 
   * DO NOT cascade deletes to Orders/OrderEntry.
   * 
   * @see edu.buffalo.cse.cse462.dao.GenericDAO#delete(java.lang.Object)
   */
  @Override
  public boolean delete(Integer key) throws DAOException {

    // the connection is shared by the session, do not close it
    Connection conn = session.getConnection();
    try {
      // this prepared statement is local, make sure you close it
      PreparedStatement stmt = conn.prepareStatement(DELETE);
      try {
        stmt.setInt(ID_CID, key);
        return stmt.executeUpdate() == 1; // exactly one tuple affected?
      }
      finally {
        stmt.close();
        stmt = null;
      }
    }
    catch (SQLException e) {
      throw new DAOException("Could not execute delete statement for customer.", e);
    }
  }

  /*
   * (non-Javadoc)
   * @see edu.buffalo.cse.cse462.dao.GenericDAO#findByKey(java.lang.Object)
   */
  @Override
  public Customer findByKey(Integer key) throws DAOException {

    // the connection is shared by the session, do not close it
    Connection conn = session.getConnection();
    try {
      // this prepared statement is local, make sure you close it
      PreparedStatement stmt = conn.prepareStatement(READ);
      try {
        stmt.setInt(ID_CID, key);
        // this result set is local, make sure you close it
        ResultSet rs = stmt.executeQuery();
        try {
          if (rs.next()) {
            return new CustomerImpl(key, rs.getString(ID_CITY), rs.getDate(ID_DOB),
                rs.getString(ID_FIRST_NAME), rs.getString(ID_LAST_NAME), rs.getString(ID_STATE));
          }
        }
        finally {
          rs.close();
          rs = null;
        }
      }
      finally {
        stmt.close();
        stmt = null;
      }
    }
    catch (SQLException e) {
      throw new DAOException("Could not execute delete statement for customer.", e);
    }
    return null;
  }

  /*
   * DO NOT cascade updates to Orders/OrderEntry.
   * 
   * (non-Javadoc)
   * @see edu.buffalo.cse.cse462.dao.GenericDAO#update(java.lang.Object)
   */
  @Override
  public boolean update(Customer updated) throws DAOException {

    // TODO: required implementation for CSE462 Project #1
    return false;
  }

  /*
   * DO NOT cascade updates to Orders/OrderEntry.
   * 
   * (non-Javadoc)
   * @see edu.buffalo.cse.cse462.dao.GenericDAO#update(java.lang.Object, java.lang.Object)
   */
  @Override
  public boolean update(Integer oldKey, Customer updated) throws DAOException {

    // TODO: required implementation for CSE462 Project #1
    return false;
  }
}